<?php
/**
* Z Content Weather Component
* @copyright (C) 2005-2012 Dan Cogliano. All rights reserved.
* www.zcontent.net
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
*/


// No direct access.
defined('_JEXEC') or die;

DEFINE ('_CACHEGROUP', 'com_zweather');
DEFINE ('_HTMLCACHEGROUP', 'com_zweather_html');
DEFINE ('_ZW_DATEFMT', 'g:ia M j, Y');
DEFINE ('_CACHEFOLDER', JPATH_SITE.DS.'cache'.DS);
//echo "cache folder: " . _CACHEFOLDER; exit;

class ZWeatherHelper extends JObject {
	var $latitude = "";
	var $longitude = "";
	var $id = "";
	var $observation_loc = "";
	var $lifetimes = array();
	
	public function __construct(){
		$this->lifetimes["fcast3"] = 3*60;
		$this->lifetimes["fcast12"] = 12*60;
		$this->lifetimes["fcast"] = 24*60;
		$this->lifetimes["observe"] = 4*60;
		$this->lifetimes["alert"] = 2*60;
		
		parent::__construct();
	}
	
	public static function getActions($extension, $id = 0)
	{
		$user		= JFactory::getUser();
		$result		= new JObject;
		$parts		= explode('.',$extension);
		$component	= $parts[0];
		$datatype	= $parts[1];

		if ($id == 0) {
			$assetName = $component;
		}
		else {
			$assetName = $component.'.'.$datatype.'.'.(int) $id;
		}

		$actions = array(
			'core.admin', 'core.manage', 'core.create', 'core.edit', 'core.edit.own', 'core.edit.state', 'core.delete'
		);

		foreach ($actions as $action) {
			$result->set($action, $user->authorise($action, $assetName));
		}

		return $result;
	}

	function isMorning($date){
		$h = date("H",$date);
		return ($h < 12);		
	}
	

	// convert NOAA date/time string to local Unix date/time value
	function convertDate( $noaadate ) {
		//$tz = substr($noaadate,19,3)*60*60 + substr($noaadate,22,2) * 60;
		$date = mktime(substr($noaadate,11,2), substr($noaadate,14,2), substr($noaadate,17,2), substr($noaadate,5,2), substr($noaadate,8,2), substr($noaadate,0,4),-1);
		$cfg = new JConfig();
		$tzname = $cfg->offset;
		$dt = date_create("@" . $date, timezone_open($tzname));
		$offset = date_offset_get($dt);
		return $date + $offset;
	}

	// convert NOAA date/time string to UTC Unix date/time value
	function convertDateUTC( $noaadate ) {
		//$tz = substr($noaadate,19,3)*60*60 + substr($noaadate,22,2) * 60;
		return mktime(substr($noaadate,11,2), substr($noaadate,14,2), substr($noaadate,17,2), substr($noaadate,5,2), substr($noaadate,8,2), substr($noaadate,0,4),-1);
	}


function getForecast3XML($options)
{
	/* Test if there is a valid cache-entry for this key */
	if ($xmldata = $this->getXMLCache($options, 'fcast3')) {
		/* Cache hit! */
	} else {
		/* Cache miss! */
		$xmldata = $this->readForecast3XML($options);
	}
	return $xmldata;
}


function getForecast12XML($options)
{
	/* Test if there is a valid cache-entry for this key */
	if ($xmldata = $this->getXMLCache($options, 'fcast12')) {
		/* Cache hit! */
	} else {
		/* Cache miss! */
		$xmldata = $this->readForecast12XML($options);
	}
	return $xmldata;
}

function getForecastXML($options)
{
	/* Test if there is a valid cache-entry for this key */
	if ($xmldata = $this->getXMLCache($options, 'fcast24')) {
		/* Cache hit! */
	} else {
		/* Cache miss! */
		$xmldata = $this->readForecastXML($options);
		//echo "(xml size: " . strlen($xmldata) . ")<br/>\n";
	}
	return $xmldata;
}

function readForecast3XML($options)
{
	$format = '3 hourly';
	$latitude = $this->latitude;
	$longitude = $this->longitude;
	$startdate = date('Y-m-d\TH:i');
	$numdays = '10';
	$url = "http://graphical.weather.gov/xml/sample_products/browser_interface/ndfdXMLclient.php?"
	. "lat=$latitude&lon=$longitude&product=time-series&begin=$startdate&temp=temp&wx=wx&icons=icons"
	. "&temp=temp&appt=appt";
	$xmldata = @file_get_contents($url);
	if(strlen($xmldata)> 0){
		$xmldata2 = explode("\n",$xmldata);
		if(substr($xmldata2[0],0,6) == '<?xml ')
			array_shift($xmldata2);
		$xmldata = implode("\n",$xmldata2);
		// save to cache
		$this->writeXMLCache($xmldata, $options, 'fcast3');
	}
	return $xmldata;
}

function readForecast12XML($options)
{
	$format = '12 hourly';
	$latitude = $this->latitude;
	$longitude = $this->longitude;
	$startdate = date('Y-m-d\TH:i');
	$enddate = date('Y-m-d\TH:i',time() + 10*24*60*60);
	$numdays = '10';
	$url = "http://graphical.weather.gov/xml/sample_products/browser_interface/ndfdBrowserClientByDay.php?"
	. "lat=$latitude&lon=$longitude&format=12+hourly&startDate=$startdate";
	//$url = "c:\Users\Dan\Downloads\weather.xml";
	$xmldata = @file_get_contents($url);
	if(strlen($xmldata)> 0){
		$xmldata2 = explode("\n",$xmldata);
		if(substr($xmldata2[0],0,6) == '<?xml ')
			array_shift($xmldata2);
		$xmldata = implode("\n",$xmldata2);
		// save to cache
		$this->writeXMLCache($xmldata, $options, 'fcast12');
	}
	//echo "xmldata: $xmldata"; exit;
	return $xmldata;
}

function readForecastXML($options)
{
	
	$format = '24 hourly';
	$latitude = $this->latitude;
	$longitude = $this->longitude;
	$startdate = date('Y-m-d\TH:i');
	$enddate = date('Y-m-d\TH:i',time() + 10*24*60*60);
	$numdays = '10';
	$url = "http://graphical.weather.gov/xml/sample_products/browser_interface/ndfdBrowserClientByDay.php?"
	. "lat=$latitude&lon=$longitude&format=24+hourly&startDate=$startdate";
	
	$xmldata = @file_get_contents($url);
	if(strlen($xmldata)> 0){
		$xmldata2 = explode("\n",$xmldata);
		if(substr($xmldata2[0],0,6) == '<?xml ')
			array_shift($xmldata2);
		$xmldata = implode("\n",$xmldata2);
		// save to cache
		$this->writeXMLCache($xmldata, $options, 'fcast24');
	}
	return $xmldata;
}

function getObservationXML($options)
{
	global $database;
	$xmldata = '';
	$cid = 'observe-' . $this->id;

	/* Test if there is a valid cache-entry for this key */
	if ($xmldata = $this->getXMLCache($options, 'observe')) {
		/* Cache hit! */
	} else {
		/* Cache miss! */
		$xmldata = $this->readObservationXML($options);
	}
	return $xmldata;
}

function readObservationXML($options)
{
	$xmldata = "";
	//echo "loading new observation XML file" . "<br/>";
	$locations = explode(",",$this->observation_loc);
	if(strlen(trim($this->observation_loc)) == 0)
		return "";
	foreach($locations as $location) {
	if(strlen($location) > 0)
	{
	$url = "http://www.weather.gov/xml/current_obs/$location.xml";
	$xmldata = @file_get_contents($url);
	if(strlen($xmldata)> 0){
		$xmldata2 = explode("\n",$xmldata);
		if(substr($xmldata2[0],0,6) == '<?xml ')
			array_shift($xmldata2);
		$xmldata = implode("\n",$xmldata2);
		// save to cache
		$this->writeXMLCache($xmldata, $options, 'observe');
	}
	return $xmldata;

	break; // stop after first location for now (future work needed)
	}
	}
	return $xmldata;
}

function getAlertXML($options)
{
	global $database;
	/* Test if there is a valid cache-entry for this key */
	if ($xmldata = $this->getXMLCache($options, 'alert')) {
		/* Cache hit! */
		//echo "alert cache hit<br/>";
	} else {
		/* Cache miss! */
		//echo "alert cache miss<br/>";
		$xmldata = $this->readAlertXML($options);
	}
	return $xmldata;
}

function readAlertXML($options)
{
	$xmldata = "";
	if(strlen(trim($this->cap_zone)) > 0){
		$czone = trim($this->cap_zone);
		$url = "http://alerts.weather.gov/cap/wwaatmget.php?x=$czone";
		$xmldata = @file_get_contents($url);
		if(strlen($xmldata)> 0){
			$xmldata2 = explode("\n",$xmldata);
			if(substr($xmldata2[0],0,6) == '<?xml ')
				array_shift($xmldata2);
			$xmldata = implode("\n",$xmldata2);
			// save to cache
			$this->writeXMLCache($xmldata, $options, 'alert');
		}
	}
	return $xmldata;
}

/*
function getLatLongFromZip($options)
{

	//echo "retrieving new zipcode XML data" . "<br/>";
	$zipcode = $options->get("zipcode");
	require_once($GLOBALS['mosConfig_absolute_path'] . '/components/com_zweather/nusoap/nusoap.php');
	$parameters = array('zipCodeList' => $zipcode);

	$ctimeout = max(0, min($options->get('ctimeout'),
			($options->get('_startsession') - time() + $options->get('maxtimeout'))));
	$timeout = max(0, min($options->get('timeout'),
			($options->get('_startsession') - time() + $options->get('maxtimeout'))));
	// Define new object and specify location of wsdl file.
	$soapclient = new nusoapclient('http://www.weather.gov/forecasts/xml/SOAP_server/ndfdXMLserver.php?wsdl',
			true, false, false, false, false, $ctimeout, $timeout);

	// Output any error conditions from creating the client
	$err = $soapclient->getError();
	//echo $err . "<br/>\n";
	if ($err)
	{
		if(intval($options->get('debug','0')) > 0){
			$this->showError("soapclient constructor error", $err);
		}
	}

	// call the method and get the result.
	//echo "calling LatLonListZipCode() with $zipcode<br/>\n";
	$xmldata = $soapclient->call('LatLonListZipCode',$parameters);
	//  Processes any SOAP fault information we get back from the server
	if ($soapclient->fault)
	{
		if(intval($options->get('debug','0')) > 0){
			$this->showError("soapclient fault", $result);
		}
	}
	else
	{
	   //  Capture any client errors
	   $err = $soapclient->getError();
	   if ($err)
	   {
		 // Display the error
		if(intval($options->get('debug','0')) > 0){
			$response = $soapclient->response;
			$this->showError("soapclient error", $err);
			if(strlen($response) > 0)
				$this->showError("HTTP response", $response);
		}
		else {
			return "";
		}
	   }
	   else  // we successfully created the DWML document
	   {
			// make sure this is a valid XML file before storing in cache
			$xmlDoc =& new DOMIT_Document();
			$xmlDoc->resolveErrors( true );
			//$xmlDoc->useHTTPClient(true);
			if (!$xmlDoc->parseXML( $xmldata, true )) {
				if(intval($options->get('debug','0')) > 0){
					$this->showError("Bad XML file(parseXML)", "");
				}
			}

			$element = $xmlDoc->documentElement;
			if ($element->getTagName() != 'dwml')
			{
				if(intval($options->get('debug','0')) > 0){
					$this->showError("Not a valid XML file", 'XML file type is ' . $element->getTagName() . ' instead of dwml.');
					return "";
				}
				else
					return "";
			}

			$xmldata =$xmlDoc->getElementsByPath('latLonList',1);
			return $xmldata->getText();
		}
	}
	return "";
}
*/

function getStations($options)
{
	global $database;
	$state = strtoupper($options->get("state"));
	$query = "select * from #__zweather_stations where state = '$state' order by station_name";
	$database->setQuery($query);
	$rows = $database->loadObjectList();
	$rowcount = count($rows);
	$htmlstr = "<table class='adminlist'>\n";
	$htmlstr .= "<tbody>\n";
	foreach($rows as $row){
		$htmlstr .= "<tr><td><a href='javascript:setStation(\"". $row->station_id . "\")'>" . $row->station_name . "</a></td><td>" . $row->station_id . "</td></tr>\n";
	}
	$htmlstr .= "</tbody>\n";
	$htmlstr .= "<thead>\n";
	$htmlstr .= "<tr><td colspan='2'>Select one of the $rowcount station(s) in this state.</td></tr>\n";
	$htmlstr .= "<tr><th>Location</th><th>Code</th></tr>\n";
	$htmlstr .= "</thead></table>\n";
	return $htmlstr;
}

// write data to cache
function writeXMLCache($data, $options, $xmltype)
{
	$cid = $xmltype . '_'. $this->id;
	$lifetime = 31536000; // set cache timeout to 1 year, using alternate method for testing cache
	$cacheoptions = array(
            'lifetime' => $lifetime,
			'caching' => 1,
			'cachebase' => _CACHEFOLDER,
	);
    $cache = new JCache($cacheoptions);
	$sdata = serialize(array(time(),$data));
    $cache->store($sdata, $cid, _CACHEGROUP);
}

// get XML cache status
// returns cache life in seconds: >0: cache current, 0: no cache, <0: stale cache
static function getXMLCacheStatus($id, $xmltype)
{
	$lifetimes= array();
	$lifetimes["fcast3"] = 60;
	$lifetimes["fcast12"] = 60;
	$lifetimes["fcast24"] = 60;
	$lifetimes["observe"] = 60;
	$lifetimes["alert"] = 10;
	
	$cid = $xmltype . '_' . $id;
	$lifetime = 31536000; // set cache timeout to 1 year, using alternate method for testing cache
	$cacheoptions = array(
		'lifetime' => $lifetime,
		'cachebase' => _CACHEFOLDER,
		'storage' => 'File'
	);
	$cache = new JCache($cacheoptions);
	$cache->setCaching(1);
	if($xmldata = $cache->get($cid, _CACHEGROUP)) {
		$lifetime = $lifetimes[$xmltype]*60;
		$xml = unserialize($xmldata);
		$life = $lifetime - time() + $xml[0];
		if($life == 0)
			$life = -1;
			return $life;
    }
    else
    	return 0;
}

// get XML file from cache
function getXMLCache($options, $xmltype)
{
	$lifetimes= array();
	$lifetimes["fcast3"] = 60;
	$lifetimes["fcast12"] = 60;
	$lifetimes["fcast24"] = 60;
	$lifetimes["observe"] = 60;
	$lifetimes["alert"] = 10;
	
	$cid = $xmltype . '_' . $this->id;
	$lifetime = 31536000; // set cache timeout to 1 year, using alternate method for testing cache
	//$lifetime = 60;
	//echo "lifetime: $lifetime";
	$cacheoptions = array(
		'lifetime' => $lifetime,
		'cachebase' => _CACHEFOLDER,
		'storage' => 'File'
	);
	$cache = new JCache($cacheoptions);
	$cache->setCaching(1);
	//echo "calling JCache::get($cid, _CACHEGROUP, " . ($checktime?"on":"off") . ")\n";
    if($xmldata = $cache->get($cid, _CACHEGROUP)) {
    	// cache hit
		$lifetime = $lifetimes[$xmltype]*60;
		if($options->get('frontendrefresh',1) == 0 && intval($options->get('cronjob',0)) == 0)
			$lifetime = 31536000; // set cache timeout to 1 year to avoid front-end from refreshing XML
		$xml = unserialize($xmldata);
    	if(time() - $xml[0] > $lifetime ||$options->get('debug_nocache') > 0){
    		// cache miss
    		return "";
    	}
    	else {
    		// cache hit
    		return $xml[1];
    	}
    	return "";
    }
    return "";
}

// this function converts an rfc822 date string to a Unix UTC timestamp (why hasn't someone written this before?)
function rfc822todate($rfc822) {
	$months = array('Jan' => 1, 'Feb' => 2, 'Mar' => 3, 'Apr' => 4,
	'May' => 5, 'Jun' => 6, 'Jul' => 7, 'Aug' => 8, 'Sep' => 9, 'Oct' => 10, 'Nov' => 11, 'Dec' => 12);
    $zones = array(
    'UT' => 0, 'GMT' => 0,
    'EDT' => -4, 'AST' => -4,
    'EST' => -5, 'CDT' => -5,
    'CST' => -6, 'MDT' => -6,
    'MST' => -7, 'PDT' => -7,
    'PST' => -8, 'YTD' => -8,
    'AKDT' => -8, 'AKST' => -9,
    'YST' => -9, 'HDT' => -9,
    'HST' => -10,'BDT' => -10);

	$month = $months[substr($rfc822, 8,3)];
	$day = substr($rfc822, 5, 2);
	$year = substr($rfc822, 12, 4);
	$hour = substr($rfc822, 17, 2);
	$minute = substr($rfc822, 20, 2);
	$second = substr($rfc822, 23, 2);
	$zone = $zones[substr($rfc822, 32)];
	$zonehour = substr($rfc822, 26,3);
	$zoneminute = substr($rfc822,29,2);
	$zonesign = ($zonehour < 0)? -1: 1;
	$zonehour = abs($zonehour);
	$localtime = mktime($hour, $minute, $second, $month, $day, $year);
	$gmttime = $localtime - $zonesign * ($zonehour*60*60 + zoneminute*60);
	return $gmttime;
}

function showError($error, $details=""){
	echo '<div class="zw_errortitle">' . $error . '</div><div class="zw_errortext">' . $details . '</div>';
}

function cleanHTMLCache() {
			$options = array(
			'cachebase' => JPATH_ROOT.DS.'cache'.DS
				);
	$cache = new Cache_Lite_Function($options);
	$cache->clean(_HTMLCACHEGROUP);
}

function cleanXMLCache() {
	$options = array(
		'cachebase' => JPATH_ROOT.DS.'cache'.DS
	);
	$cache = new Cache_Lite_Function($options);
	$cache->clean(_CACHEGROUP);
}

function cleanCache() {
			$options = array(
				'cachebase' => JPATH_ROOT.DS.'cache'.DS
				);
	$cache = new Cache_Lite_Function($options);
	$cache->clean(_HTMLCACHEGROUP);
	$cache->clean(_CACHEGROUP);
}

function isToday($checkdate)
{
	if ( date('Y-m-d', $checkdate) == date('Y-m-d', time()) )
	{
		return true;
	}
	return false;
}

function isTomorrow($checkdate)
{
	if ( date('Y-m-d', $checkdate) == date('Y-m-d', time() + 60 * 60 * 24) )
	{
		return true;
	}
	return false;
}

//date math: add to current date to get new date
function addDate($date, $hour, $min, $sec, $month, $day, $year, $dst = -1) {
	$tdate=getdate($date);
	$newdate=mktime($tdate["hours"] + $hour, $tdate["minutes"] + $min, $tdate["seconds"] + $sec, $tdate["mon"] + $month, $tdate["mday"] + $day, $tdate["year"] + $year, $dst);
	return $newdate;
}

}



/*
* This class is used by the cronjob feature
*/
class josZWeatherCronJob {

// This version of showError does not create HTML code
function showError($error, $details=""){
	echo '**** Error: ' . $error;
	if(strlen($details) > 0) {
		echo ": " . $details;
	}
	echo "\n";
}

}

?>
